/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.analysis.*;

import jalview.datamodel.*;

import jalview.io.*;

import jalview.jbgui.*;


import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.*;

import java.beans.PropertyChangeEvent;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision$
 */
public class TreePanel extends GTreePanel
{
    SequenceI[] seq;
    String type;
    String pwtype;
    int start;
    int end;
    TreeCanvas treeCanvas;
    NJTree tree;

    /**
     * Creates a new TreePanel object.
     *
     * @param av DOCUMENT ME!
     * @param seqVector DOCUMENT ME!
     * @param type DOCUMENT ME!
     * @param pwtype DOCUMENT ME!
     * @param s DOCUMENT ME!
     * @param e DOCUMENT ME!
     */
    public TreePanel(AlignViewport av, Vector seqVector, String type,
                     String pwtype, int s, int e)
    {
      super();
      initTreePanel(av, seqVector, type, pwtype, s, e, null);

      // We know this tree has distances. JBPNote TODO: prolly should add this as a userdefined default
      showDistances(true);
    }

    /**
     * Creates a new TreePanel object.
     *
     * @param av DOCUMENT ME!
     * @param seqVector DOCUMENT ME!
     * @param newtree DOCUMENT ME!
     * @param type DOCUMENT ME!
     * @param pwtype DOCUMENT ME!
     */
    public TreePanel(AlignViewport av, Vector seqVector, NewickFile newtree,
                     String type, String pwtype)
    {
      super();
      initTreePanel(av, seqVector, type, pwtype, 0, seqVector.size(), newtree);
    }

    public AlignmentI getAlignment()
    {
      return treeCanvas.av.getAlignment();
    }


    void initTreePanel(AlignViewport av, Vector seqVector, String type,
                       String pwtype, int s, int e, NewickFile newTree)
    {

      this.type = type;
      this.pwtype = pwtype;

      start = s;
      end = e;

      seq = new Sequence[seqVector.size()];
      seqVector.toArray(seq);


      treeCanvas = new TreeCanvas(av, scrollPane);
      scrollPane.setViewportView(treeCanvas);

      av.addPropertyChangeListener(new java.beans.PropertyChangeListener()
      {
        public void propertyChange(PropertyChangeEvent evt)
        {
          if (evt.getPropertyName().equals("alignment"))
          {
            if(tree==null)
              System.out.println("tree is null");
            if(evt.getNewValue()==null)
              System.out.println("new value is null");

            tree.UpdatePlaceHolders( (Vector) evt.getNewValue());

            repaint();
          }
        }
      });


      TreeLoader tl = new TreeLoader(newTree);
      tl.start();

    }

    class TreeLoader extends Thread
    {
      NewickFile newtree;

      public TreeLoader(NewickFile newtree)
      {
        this.newtree = newtree;
        if (newtree != null)
        {
          // Must be outside run(), as Jalview2XML tries to
          // update distance/bootstrap visibility at the same time
          showBootstrap(newtree.HasBootstrap());
          showDistances(newtree.HasDistances());
          showPlaceholders(true);
        }
      }

      public void run()
      {
        if(newtree!=null)
          tree = new NJTree(seq, newtree);
        else
          tree = new NJTree(seq, type, pwtype, start, end);

        tree.reCount(tree.getTopNode());
        tree.findHeight(tree.getTopNode());
        treeCanvas.setTree(tree);

        treeCanvas.repaint();


        fullid.setState(treeCanvas.av.showDBPrefix);
        if(!treeCanvas.av.showDBPrefix)
        {
          fullid_actionPerformed(null);
        }
      }

    }

    public void showDistances(boolean b)
    {
      treeCanvas.setShowDistances(b);
      distanceMenu.setSelected(b);
    }

    public void showBootstrap(boolean b)
    {
      treeCanvas.setShowBootstrap(b);
      bootstrapMenu.setSelected(b);
    }

    public void showPlaceholders(boolean b)
    {
      placeholdersMenu.setState(b);
      treeCanvas.setMarkPlaceholders(b);
    }




    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public NJTree getTree()
    {
        return tree;
    }


    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void textbox_actionPerformed(ActionEvent e)
    {
        CutAndPasteTransfer cap = new CutAndPasteTransfer();

        StringBuffer buffer = new StringBuffer();

        if (type.equals("AV"))
        {
            buffer.append("Average distance tree using ");
        }
        else
        {
            buffer.append("Neighbour joining tree using ");
        }

        if (pwtype.equals("BL"))
        {
            buffer.append("BLOSUM62");
        }
        else
        {
            buffer.append("PID");
        }

        Desktop.addInternalFrame(cap, buffer.toString(), 500, 100);

        jalview.io.NewickFile fout = new jalview.io.NewickFile(tree.getTopNode());
        cap.setText(fout.print(false, true));
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void saveAsNewick_actionPerformed(ActionEvent e)
    {
        JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty(
                    "LAST_DIRECTORY"));
        chooser.setFileView(new JalviewFileView());
        chooser.setDialogTitle("Save tree as newick file");
        chooser.setToolTipText("Save");

        int value = chooser.showSaveDialog(null);

        if (value == JalviewFileChooser.APPROVE_OPTION)
        {
            String choice = chooser.getSelectedFile().getPath();
            jalview.bin.Cache.setProperty("LAST_DIRECTORY",
                chooser.getSelectedFile().getParent());

            try
            {
                jalview.io.NewickFile fout = new jalview.io.NewickFile(tree.getTopNode());
                String output = fout.print(false, true); // distances only
                java.io.PrintWriter out = new java.io.PrintWriter(new java.io.FileWriter(
                            choice));
                out.println(output);
                out.close();
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void printMenu_actionPerformed(ActionEvent e)
    {
        //Putting in a thread avoids Swing painting problems
        treeCanvas.startPrinting();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void fitToWindow_actionPerformed(ActionEvent e)
    {
        treeCanvas.fitToWindow = fitToWindow.isSelected();
        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void font_actionPerformed(ActionEvent e)
    {
        if (treeCanvas == null)
        {
            return;
        }

        new FontChooser(this);
    }

    public Font getTreeFont()
    {
        return treeCanvas.font;
    }

    public void setTreeFont(Font font)
    {
      if(treeCanvas!=null)
      treeCanvas.setFont(font);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void distanceMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setShowDistances(distanceMenu.isSelected());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void bootstrapMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setShowBootstrap(bootstrapMenu.isSelected());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void placeholdersMenu_actionPerformed(ActionEvent e)
    {
        treeCanvas.setMarkPlaceholders(placeholdersMenu.isSelected());
    }


    public void fullid_actionPerformed(ActionEvent e)
    {
      tree.findHeight(tree.getTopNode());

      // Now have to calculate longest name based on the leaves
      Vector leaves = tree.findLeaves(tree.getTopNode(), new Vector());

      treeCanvas.longestName="";
      for (int i = 0; i < leaves.size(); i++)
      {
        SequenceNode lf = (SequenceNode) leaves.elementAt(i);
        Sequence seq = (Sequence) lf.element();
        lf.setName(seq.getDisplayId(fullid.getState(), false));
        if(lf.getName().length() > treeCanvas.longestName.length())
          treeCanvas.longestName = lf.getName();
      }

      treeCanvas.repaint();
    }


    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void epsTree_actionPerformed(ActionEvent e)
    {
      makeTreeImage(jalview.util.ImageMaker.EPS);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void pngTree_actionPerformed(ActionEvent e)
    {
       makeTreeImage(jalview.util.ImageMaker.PNG);
    }

    void makeTreeImage(int type)
    {
      int width = treeCanvas.getWidth();
      int height = treeCanvas.getHeight();

      jalview.util.ImageMaker im;

      if(type == jalview.util.ImageMaker.PNG)
        im = new jalview.util.ImageMaker(this,
                                         jalview.util.ImageMaker.PNG,
                                         "Make PNG image from tree",
                                         width, height,
                                         null, null);
        else
          im = new jalview.util.ImageMaker(this,
                                 jalview.util.ImageMaker.EPS,
                                 "Make EPS file from tree",
                                 width, height,
                                 null, this.getTitle());

      if(im.getGraphics()!=null)
      {
        treeCanvas.draw(im.getGraphics(), width, height);
        im.writeImage();
      }
    }

}
